@@ -186,10 +186,6 @@ class _GenericAdvFuncHelper:
     def sql_current_timestamp(self):
         return 'CURRENT_TIMESTAMP'
 
-    def sql_create_sequence(self, seq_name):
-        return '''CREATE TABLE %s (last INTEGER);
-INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
-
     def sql_create_index(self, table, column, unique=False):
         idx = self._index_name(table, column, unique)
         if unique:
@@ -204,6 +200,10 @@ INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
         else:
             return 'DROP INDEX %s' % idx
 
+    def sql_create_sequence(self, seq_name):
+        return '''CREATE TABLE %s (last INTEGER);
+INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
+
     def sql_drop_sequence(self, seq_name):
         return 'DROP TABLE %s;' % seq_name
 
@@ -230,6 +230,9 @@ INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
         else:
             return 'FALSE'
 
+    def binary_value(self, value):
+        return value
+
     def increment_sequence(self, cursor, seq_name):
         for sql in self.sqls_increment_sequence(seq_name):
             cursor.execute(sql)
@@ -566,11 +569,36 @@ class _MyAdvFuncHelper(_GenericAdvFuncHelper):
             allindices += self.list_indices(cursor, table)
         return allindices
 
+class _SqlServer2005FuncHelper(_GenericAdvFuncHelper):
+    backend_name = 'sqlserver2005'
+    ilike_support = False
+    # modifiable but should not be shared
+    FUNCTIONS = _GenericAdvFuncHelper.FUNCTIONS.copy()
+    TYPE_MAPPING = {
+        'String' :   'ntext',
+        'Int' :      'integer',
+        'Float' :    'float',
+        'Decimal' :  'decimal',
+        'Boolean' :  'bit',
+        'Date' :     'smalldatetime',
+        'Time' :     'time',
+        'Datetime' : 'datetime',
+        'Interval' : 'interval',
+        'Password' : 'varbinary(255)',
+        'Bytes' :    'varbinary(max)',
+        }
+
+    def list_tables(self, cursor):
+        """return the list of tables of a database"""
+        return  [row.table_name for row in cursor.tables()]
+    def binary_value(self, value):
+        return StringIO.StringIO(value)
 
 
 ADV_FUNC_HELPER_DIRECTORY = {'postgres': _PGAdvFuncHelper(),
                              'sqlite': _SqliteAdvFuncHelper(),
                              'mysql': _MyAdvFuncHelper(),
+                             'sqlserver2005': _SqlServer2005FuncHelper(),
                              }
 
 
